#
# Utility functions used by API requests
#
import sys
import logging
import re
import json
import ipaddress

from barix.system import *
sys.path.append("/barix/apps/ipac/")
from barix.system.barix_enums import BarixHWType

log = logging.getLogger('webui-backend')

MAX_AUDIO_SOURCES = 5

def validateSourcesConfiguration(sourcesCfg):
    try:
        setDefaultEqualSourcesList = []

        if "sources" not in sourcesCfg:
            return False, "Invalid sources json config file"

        for source_key in sourcesCfg['sources']:

            if "source" not in source_key:
                return False, f"Invalid source key name: {source_key}"
            if int(source_key.split("source")[1]) < 1 or int(source_key.split("source")[1]) > MAX_AUDIO_SOURCES:
                return False, f"Source id {source_key.split('source')[1]} out of range"

            if 'enabled' not in sourcesCfg["sources"][source_key]:
                return False, f'Missing "enabled" attribute on source "{source_key}"'

            if not sourcesCfg["sources"][source_key]['enabled']:
                continue

            if len(sourcesCfg["sources"][source_key]) == 0:
                log.warning(f"{source_key} with empty config ... move on to the next")
                continue
            valid, description = validateSourceCommonParameters(sourcesCfg["sources"][source_key])
            if not valid:
                return valid, f"{source_key}: {description}"

            # check source specific config
            srcType = sourcesCfg['sources'][source_key]['type']
            srcConfig = sourcesCfg['sources'][source_key]['config']

            if srcType == 'informacast':
                if not informacastValidLicense():
                    return False, f"{source_key}: Cannot configure informacast source - license required!"
                valid, description = checkInformacastSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'syn-apps':
                valid, description = checkSynappsSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'barix_radio':
                checkBarixRadioSource(srcConfig)
                pass
            elif srcType == 'rtp':
                valid, description = checkRtpSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'local_file':
                valid, description = checkLocalFileSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'sip':
                valid, description = checkSipSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'http':
                valid, description = checkHttpSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'paging':
                valid, description = checkPagingSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'usb':
                valid, description = checkUsbSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'scheduler':
                valid, description = checkSchedulerSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'line-in':
                pass
            elif srcType == 'udp':
                valid, description = checkUdpSource(srcConfig)
                if not valid:
                    return valid, f"{source_key}: {description}"
            elif srcType == 'dante':
                pass
            else:
                return False, f"{source_key}: Invalid source type:{srcType}"

            # Check equalizer config (all sources except: Informacast, Syn-apps and SIP)
            if srcType != 'informacast' and srcType != 'syn-apps' and srcType != 'sip':
                if "equalizer" in srcConfig:
                    valid, description = checkEqualizerConfig(srcConfig)
                    if not valid:
                        return valid, f"{source_key}: {description}"
                else:
                    setDefaultEqualSourcesList.append(source_key)

        return True, ""

    except Exception as e:
        log.error(e)
        return False, "Sources configuration error"

def validateSourceCommonParameters(source):
    if 'name' not in source:
        return False, "Missing source name key"
    if "type" not in source:
        return False, "Missing source type key"
    if "config" not in source:
        return False, "Missing config type key"
    if "volume" not in source:
        return False, "Missing source volume key"
    if "prio" not in source:
        return False, "Missing prio volume key"
    if "enabled" not in source:
        return False, "Missing enabled volume key"
    vol = source['volume']
    if type(vol) != int:
        return False, f"Volume '{vol}' is not an integer"
    if vol < 0 or vol > 100:
        return False, f"Volume '{vol}' is out of range - must be between 0 and 100"
    prio = source['prio']
    if type(prio) != int:
        return False, f"Prio '{prio} 'is not an integer"
    if prio < 0 or prio > 5:
        return False, f"Prio value {prio} is out of range - must be between 1 and 5"
    enabled = source['enabled']
    if type(enabled) != bool:
        return False, f"Enabled '{enabled}' is not boolean"
    return True, ""


def informacastValidLicense():
    with open("/tmp/factory_info_partition/factory_info_json") as jsonFile:
        config_json = json.load(jsonFile)
        for result in config_json['SW_LICENSES']:
            if re.search('InformaCast', result['Image_Features']):
                return True
    return False

def checkInformacastSource(srcCfg):
    if "url" not in srcCfg:
        return False, "Missing url key in informacast config"
    if type(srcCfg['url']) != str:
        return False, "Invalid url in informacast config"
    match = re.match(r'^http[s]?:\/\/([0-9]|[1-8][0-9]|9[0-9]|1[0-9]{2}|2[01][0-9]|22[0-3])(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}(:([1-9]|[1-8][0-9]|9[0-9]|[1-8][0-9]{2}|9[0-8][0-9]|99[0-9]|[1-8][0-9]{3}|9[0-8][0-9]{2}|99[0-8][0-9]|999[0-9]|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$|^http[s]?:\/\/(([a-zA-Z]{1})|([a-zA-Z]{1}[a-zA-Z]{1})|([a-zA-Z]{1}[0-9]{1})|([0-9]{1}[a-zA-Z]{1})|([a-zA-Z0-9][a-zA-Z0-9-_]{1,61}[a-zA-Z0-9]))\.(([a-zA-Z0-9-]{1,30}\.){1,2}[a-zA-Z]{2,3})(:([1-9]|[1-8][0-9]|9[0-9]|[1-8][0-9]{2}|9[0-8][0-9]|99[0-9]|[1-8][0-9]{3}|9[0-8][0-9]{2}|99[0-8][0-9]|999[0-9]|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$|^$',srcCfg['url'])
    if not match:
        return False, "Invalid url in informacast config"
    if srcCfg['url'] != '':
        if "idleaudio" not in srcCfg:
            return False, "Missing idleaudio key in informacast config"
        if type(srcCfg['idleaudio']) != str:
            return False, "Invalid idleaudio in informacast config"
        if srcCfg['idleaudio'] != '':
            match = re.match(r'^rtp:\/\/((2(?:2[4-9]|3\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}:)|@:|0.0.0.0:)(102[5-9]|10[3-9][0-9]|1[1-9][0-9]{2}|[2-9][0-9]{3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])?$',srcCfg['idleaudio'])
            if not match:
                return False, "Invalid idleaudio url format in informacast config"
        if "idleaudio_format" not in srcCfg:
            return False, "Missing idleaudio_format key in informacast config"
        if srcCfg['idleaudio_format'] not in ["ULAW_8KHZ", "PCM_44KHZ"]:
            return False, f"Invalid or missing idle audio format: {srcCfg['idleaudio_format']}"
    return True, ""

def checkSynappsSource(srcCfg):
    if "server" not in srcCfg:
        return False, "Missing server key in synapps config"
    if type(srcCfg['server']) != str:
        return False, "Invalid server in synapps config"
    match = re.match(r'^([0-9]|[1-8][0-9]|9[0-9]|1[0-9]{2}|2[01][0-9]|22[0-3])(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}$|^$',srcCfg['server'])
    if not match:
        return False, "Invalid server in synapps config"
    return True, ""

def checkBarixRadioSource(srcCfg):
    pass

def checkLocalFileSource(srcCfg):
    if "port" not in srcCfg:
        return False, "Missing port key in local file config"
    if int(srcCfg['port']) < 8000 or int(srcCfg['port']) > 8999:
        return False, "Invalid port number in local file config - range: 8000 to 8999"
    if "loop" not in srcCfg:
        return False, "Missing loop key in local file config"
    if type(srcCfg['loop']) != bool :
        return False, "Invalid local config loop value"
    return True, ""

def checkRtpSource(srcCfg):
    if "url" not in srcCfg:
        return False, "Missing url key in rtp config"
    if "sdp" in srcCfg and srcCfg['sdp']!="" and not validSdp(srcCfg['sdp']) :
        return False, "Invalid sdp in rtp config"
    if srcCfg['url'] == "":
        if "sdp" not in srcCfg: # comming from remote API
            return False, "Invalid url in rtp config"
        if srcCfg["sdp"] == "":
            return False, "Invalid url in rtp config"
    else:
        match = re.match(r'^rtp:\/\/((2(?:2[4-9]|3\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}:)|@:|0.0.0.0:)(102[5-9]|10[3-9][0-9]|1[1-9][0-9]{2}|[2-9][0-9]{3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(\?format=\w+&channels=\d&rate=\d+(&pt=\d+)?)?$', srcCfg['url'])
        if not match:
            return False, "Invalid rtp url"
    if "playback_mode" not in srcCfg:
        return False, "Missing playback_mode key in rtp config"
    if srcCfg['playback_mode'] not in ["low_latency", "regular"]:
        return False, "Invalid rtp playback mode"
    return True, ""

def validSdp(sdpCfg):
    if type(sdpCfg) == str:
        sdpLines = []
        if "\r\n" in sdpCfg:
            sdpLines = sdpCfg.split('\r\n')
        else:
            sdpLines = sdpCfg.split('\n')
        if len(sdpLines) > 0:
            for line in sdpLines:
                if not re.match(r'^.+=.+$', line):
                    return False
            return True
    return False

def checkPagingSource(srcCfg):
    if "paging_mode" not in srcCfg:
        return False, "Missing paging_mode key in paging config"
    if srcCfg['paging_mode'] not in ["simple paging", "ic paging"]:
        return False, "Invalid paging mode"

    if "barp_station_id" not in srcCfg:
        return False, "Missing barp_station_id key in paging config"
    if type(srcCfg['barp_station_id']) != int:
        return False, f"barp_station_id {srcCfg['barp_station_id']} is not an integer"
    if srcCfg['barp_station_id'] < 1 or srcCfg['barp_station_id'] > 128:
        return False, f"barp_station_id {srcCfg['barp_station_id']} is out of range - must be between 1 to 128"

    if srcCfg['paging_mode'] == "ic paging":
        if "barp_address" not in srcCfg:
            return False, "Missing barp_address key in paging config"
        if "barp_address" == "":
            return False, "Missing bar_address in paging config"
        match = re.match(r'^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$',srcCfg['barp_address'])
        if not match:
            return False, f"barp_address {srcCfg['barp_address']} is invalid"
        if srcCfg['barp_address'] != '0.0.0.0' and not ipaddress.IPv4Address(srcCfg['barp_address']).is_multicast:
            return False, f"barp_address {srcCfg['barp_address']} is multicast"

        if "barp_audio_port" not in srcCfg:
            return False, "Missing barp_audio_port key in paging config"
        if type(srcCfg['barp_audio_port']) != int:
            return False, f"barp_audio_port {srcCfg['barp_audio_port']} is not an integer"
        if srcCfg['barp_audio_port'] < 5555 or srcCfg['barp_audio_port'] > 5999:
            return False, f"barp_audio_port {srcCfg['barp_audio_port']} is out of range - must be between 5555 to 5999"

        if "barp_control_port" not in srcCfg:
            return False, "Missing barp_control_port key in paging config"
        if type(srcCfg['barp_control_port']) != int:
            return False, f"barp_control_port {srcCfg['barp_control_port']} is not an integer"
        if srcCfg['barp_control_port'] < 5555 or srcCfg['barp_control_port'] > 5999:
            return False, f"barp_control_port {srcCfg['barp_control_port']} is out of range - must be between 5555 to 5999"

        if "barp_status_port" not in srcCfg:
            return False, "Missing barp_status_port key in paging config"
        if type(srcCfg['barp_status_port']) != int:
            return False, f"barp_status_port {srcCfg['barp_status_port']} is not an integer"
        if srcCfg['barp_status_port'] < 5555 or srcCfg['barp_status_port'] > 5999:
            return False, f"barp_status_port {srcCfg['barp_status_port']} is out of range - must be between 5555 to 5999"

        if "sample_rate" not in srcCfg:
            return False, "Missing sample_rate key in paging config"
        if type(srcCfg['sample_rate']) != int:
            return False, f"sample_rate {srcCfg['sample_rate']} is not an integer"
        if srcCfg['sample_rate'] != 8000 and srcCfg['sample_rate'] != 24000:
            return False, f"sample_rate {srcCfg['sample_rate']} is invalid, must be 8000 or 24000"

        if "audio_format" not in srcCfg:
            return False, "Missing audio_format key in paging config"
        audioFormats = ["mulaw", "alaw"]
        if srcCfg['audio_format'] not in audioFormats:
            return False, "Invalid audio format. Possible options: mulaw or alaw"

    return True, ""

def checkSipSource(srcCfg):
    if "sip_mode" not in srcCfg:
        return False, "Missing sip_mode key in sip config"
    sipModes = ["peer_to_peer", "server", "server_and_peer_to_peer"]
    if srcCfg['sip_mode'] not in sipModes:
        return False, "Invalid sip_mode in sip config"

    # common parameters for all sip modes
    if "snd_play_latency" not in srcCfg:
        return False, "Missing snd_play_latency key (Playback buffer) in sip config"
    if type(srcCfg['snd_play_latency']) != int:
        return False, f"snd_play_latency {srcCfg['snd_play_latency']} is not an integer"
    if srcCfg['snd_play_latency'] < 5 or srcCfg['snd_play_latency'] > 200:
        return False, f"snd_play_latency {srcCfg['snd_play_latency']} is out of range - must be between 5 to 200"

    if "snd_rec_latency" not in srcCfg:
        return False, "Missing snd_rec_latency key (Capture buffer) in sip config"
    if type(srcCfg['snd_rec_latency']) != int:
        return False, f"snd_rec_latency {srcCfg['snd_rec_latency']} is not an integer"
    if srcCfg['snd_rec_latency'] < 5 or srcCfg['snd_rec_latency'] > 200:
        return False, f"snd_rec_latency {srcCfg['snd_rec_latency']} is out of range - must be between 5 to 200"

    if "call_timeout" not in srcCfg:
        return False, "Missing call_timeout key in sip config"
    if type(srcCfg['call_timeout']) != int:
        return False, f"call_timeout {srcCfg['call_timeout']} is not an integer"
    if srcCfg['call_timeout'] < 0:
        return False, f"call_timeout {srcCfg['call_timeout']} is out of range - must be 0 or greater"

    if "stream_timeout" not in srcCfg:
        return False, "Missing stream_timeout key in sip config"
    if type(srcCfg['stream_timeout']) != int:
        return False, f"stream_timeout {srcCfg['stream_timeout']} is not an integer"
    if srcCfg['stream_timeout'] < 0:
        return False, f"stream_timeout {srcCfg['stream_timeout']} is out of range - must be 0 or greater"

    if "auto_answer" not in srcCfg:
        return False, "Missing auto_answer key in sip config"
    if type(srcCfg['auto_answer']) != bool:
        return False, "Invalid auto_answer - must be boolean"

    if "answer_time" not in srcCfg:
        return False, "Missing answer_time key in sip config"
    if type(srcCfg['answer_time']) != int:
        return False, f"answer_time {srcCfg['answer_time']} is not an integer"
    if srcCfg['answer_time'] < 0 or srcCfg['answer_time'] > 300:
        return False, f"answer_time {srcCfg['answer_time']} is out of range - must be between 0 to 300"

    if "input_gain" not in srcCfg:
        return False, "Missing input_gain key in sip config"
    if type(srcCfg['input_gain']) != int:
        return False, f"input_gain {srcCfg['input_gain']} is not an integer"
    if srcCfg['input_gain'] < 0 or srcCfg['input_gain'] > 100:
        return False, f"input_gain {srcCfg['input_gain']} is out of range - must be between 0 to 100"

    if "half_duplex" not in srcCfg:
        return False, "Missing half_duplex key in sip config"
    if type(srcCfg['half_duplex']) != bool:
        return False, "Invalid half_duplex - must be boolean"

    if "hdx_level" not in srcCfg:
        return False, "Missing hdx_level key in sip config"
    if type(srcCfg['hdx_level']) != float:
        return False, f"hdx_level {srcCfg['hdx_level']} is not an float"
    if srcCfg['hdx_level'] < 0.0 or srcCfg['hdx_level'] > 1.0:
        return False, f"hdx_level {srcCfg['hdx_level']} is out of range - must be between 0.0 to 1.0"

    if "hdx_timeout" not in srcCfg:
        return False, "Missing hdx_timeout key in sip config"
    if type(srcCfg['hdx_timeout']) != int:
        return False, f"hdx_timeout {srcCfg['hdx_timeout']} is not an integer"
    if srcCfg['hdx_timeout'] < 10 or srcCfg['hdx_timeout'] > 1000:
        return False, f"hdx_timeout {srcCfg['hdx_timeout']} is out of range - must be between 10 to 1000"

    if "port" in srcCfg:  # optional parameter
        if type(srcCfg['port']) != int:
            return False, f"SIP port {srcCfg['port']} is not an integer"
        if srcCfg['port'] < 0 or srcCfg['port'] > 65535:
            return False, f"SIP port {srcCfg['port']} is out of range - must be between 0 to 65535"

    # specific parameters for each server/server_and_peer_to_peer sip modes
    if srcCfg['sip_mode'] == "server" or srcCfg['sip_mode'] == "server_and_peer_to_peer":
        if "domain" not in srcCfg:
            return False, "Missing domain key in sip config"

        if "user_name" not in srcCfg:
            return False, "Missing user_name key in sip config"
        if type(srcCfg['user_name']) != str:
            return False, "Invalid user name in sip config"
        if srcCfg['user_name'] == "":
            return False, "Invalid user name in sip config"

        if "passwd" not in srcCfg:
            return False, "Missing passwd key in sip config"
        if type(srcCfg['passwd']) != str:
            return False, "Invalid passwd in sip config"

        if "transport_and_encryption" not in srcCfg:
            return False, "Missing transport_and_encryption key in sip config"
        transportEncryptionValues = ["udp", "tls", "tls_srtp", "tcp"]
        if srcCfg['transport_and_encryption'] not in transportEncryptionValues:
            return False, "Invalid transport/encryption in sip config"

        if "reg_timeout" not in srcCfg:
            return False, "Missing reg_timeout key in sip config"
        if type(srcCfg['reg_timeout']) != int:
            return False, f"reg_timeout {srcCfg['reg_timeout']} is not an integer"
        if srcCfg['reg_timeout'] < 5 or srcCfg['reg_timeout'] > 300:
            return False, f"reg_timeout {srcCfg['reg_timeout']} is out of range - must be between 5 to 300"

        if "reg_retry_interval" not in srcCfg:
            return False, "Missing reg_retry_interval key in sip config"
        if type(srcCfg['reg_retry_interval']) != int:
            return False, f"reg_retry_interval {srcCfg['reg_retry_interval']} is not an integer"
        if srcCfg['reg_retry_interval'] < 0:
            return False, f"reg_retry_interval {srcCfg['reg_retry_interval']} is out of range - must be 0 or greater"

    return True, ""

def checkUsbSource(srcCfg):
    #TPA does not support USB
    if int(get_hw_type()) == BarixHWType.Barix_TPA400.value:
        return False, "Cannot configure usb audio source in TPA device"
    if "path" not in srcCfg:
        return False, "Missing path key in usb config"
    if type(srcCfg['path']) != str:
        return False, "Invalid path in sip config"
    return True, ""

def checkHttpSource(srcCfg):
    if "url" not in srcCfg:
        return False, "Missing url key in http config"
    if type(srcCfg['url']) != str:
        return False, "Invalid url in http config"
    if srcCfg['url'] == "":
        return False, "Missing url in http config"
    #match = re.match(r'^http[s]?:\/\/(.*:.*@)?([0-9]|[1-8][0-9]|9[0-9]|1[0-9]{2}|2[01][0-9]|22[0-3])(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}(:([1-9]|[1-8][0-9]|9[0-9]|[1-8][0-9]{2}|9[0-8][0-9]|99[0-9]|[1-8][0-9]{3}|9[0-8][0-9]{2}|99[0-8][0-9]|999[0-9]|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$|^http[s]?:\/\/(.*:.*@)?(([a-zA-Z]{1})|([a-zA-Z]{1}[a-zA-Z]{1})|([a-zA-Z]{1}[0-9]{1})|([0-9]{1}[a-zA-Z]{1})|([a-zA-Z0-9][a-zA-Z0-9-_]{1,61}[a-zA-Z0-9]))\.(([a-zA-Z0-9-]{1,30}\.)*[a-zA-Z]{2,3})(:([1-9]|[1-8][0-9]|9[0-9]|[1-8][0-9]{2}|9[0-8][0-9]|99[0-9]|[1-8][0-9]{3}|9[0-8][0-9]{2}|99[0-8][0-9]|999[0-9]|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$',srcCfg['url'])
    match = re.match(r'^http[s]?:\/\/(.+)$',srcCfg['url'])
    if not match:
        return False, "Invalid url in http config"
    return True, ""

def checkSchedulerSource(scrCfg):
    return True, ""

def checkUdpSource(srcCfg):
    if "url" not in srcCfg:
        return False, "Missing url key in udp config"
    if srcCfg['url'] == "":
        return False, "Invalid url in udp config"
    match = re.match(r'^udp:\/\/((2(?:2[4-9]|3\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]\d?|0)){3}:)|@:|0.0.0.0:)(102[5-9]|10[3-9][0-9]|1[1-9][0-9]{2}|[2-9][0-9]{3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])(\?format=\w+&channels=\d&rate=\d+(&pt=\d+)?)?$', srcCfg['url'])
    if not match:
        return False, "Invalid udp url"

    if "sample_rate" not in srcCfg:
        return False, "Missing sample_rate key in udp config"
    if type(srcCfg['sample_rate']) != int:
        return False, f"sample_rate {srcCfg['sample_rate']} is not an integer"
    if srcCfg['sample_rate'] != 8000 and srcCfg['sample_rate'] != 24000:
        return False, f"sample_rate {srcCfg['sample_rate']} is invalid, must be 8000 or 24000"

    if "audio_format" not in srcCfg:
        return False, "Missing audio_format key in udp config"
    audioFormats = ["mulaw", "alaw"]
    if srcCfg['audio_format'] not in audioFormats:
        return False, "Invalid audio format. Possible options: mulaw or alaw"
    return True, ""

def checkEqualizerConfig(srcCfg):
    if "enable" not in srcCfg['equalizer']:
        return False, "Missing enable key parameter in equalizer"
    if type(srcCfg['equalizer']['enable']) != bool:
        return False, "Invalid enable value in equalizer"

    if "config" not in srcCfg['equalizer']:
        return False, "Missing config key parameter in equalizer"

    if len(srcCfg['equalizer']['config']) != 4: #must have 4 band filter config
        return False, "Invalid equalizer config - must have 4 band filter config"
    filterNameCheckList = []
    filterName = ["LOW", "LOW-MID", "HI-MID", "HI"]
    for filter in srcCfg['equalizer']['config']:
        if "name" not in filter:
            return False, "Invalid equalizer config - missing filter parameter 'name'"
        if "f" not in filter:
            return False, "Invalid equalizer config - missing filter parameter 'f'"
        if "t" not in filter:
            return False, "Invalid equalizer config - missing filter parameter 't'"
        if "w" not in filter:
            return False, "Invalid equalizer config - missing filter parameter 'w'"
        if "g" not in filter:
            return False, "Invalid equalizer config - missing filter parameter 'g'"

        if filter['name'] not in filterName:
            return False, f"Invalid equalizer filter name: {filter['name']}"
        if filter['name'] in filterNameCheckList:
            return False, f"Equalizer filter {filter['name']} is used more than once"
        filterNameCheckList.append(filter['name'])

        if filter['t'] != 'q':
            return False, "Invalid with type 't' value, must be 'q' (q-factor)"

        f = int(filter['f'])
        if filter['name'] == "LOW":
            if f < 20 or f > 400:
                return False, f"Invalid equalizer {filter['name']} filter central frequency: {f} - range: 20 to 400 (Hz)"
        if filter['name'] == "LOW-MID":
            if f < 80 or f > 1000:
                return False, f"Invalid equalizer {filter['name']} filter central frequency: {f} - range: 80 to 1000 (Hz)"
        if filter['name'] == "HI-MID":
            if f < 800 or f > 12500:
                return False, f"Invalid equalizer {filter['name']} filter central frequency: {f} - range: 800 to 12500 (Hz)"
        if filter['name'] == "HI":
            if f < 2500 or f > 20000:
                return False, f"Invalid equalizer {filter['name']} filter central frequency: {f} - range: 2500 to 20000 (Hz)"

        w = float(filter['w'])
        if w < 0.1 or w > 8:
            return False, f"Invalid equalizer filter q factor: {w} - range: 0.1 to 8"
        g = int(filter['g'])
        if g < -24 or g > 24:
            return False, f"Invalid equalizer filter gain: {g} - range: -24 to 24 (dB)"

    return True, ""

# ----------------------------------

def getFilename(fileId):
    """
    Retrieve filename from file id
    :param fileId: audio file id
    :return: filename
    """
    try:
        res = ""
        with open("/mnt/data/config/audioFiles.json") as f:
            data = json.load(f)
            for index in data:
                if index["file_id"] == fileId:
                    return index["filename"]
        return res
    except Exception as e:
        log.error(f"getFilename Exception {e}")
        return res
