#!/usr/bin/python3

# =================================================================
# Set IPAC specific DEP settings depending on the application mode
#
# =================================================================

import sys
import syslog
import os
import json
import shutil
import logging
from uci import Uci
from barix.system import get_hw_type
from barix.system.barix_enums import BarixHWType

logger = logging.getLogger(__name__)

logging.basicConfig(filename='/var/log/set_dep_config.log', level=logging.INFO, format='%(asctime)s %(name)s %(levelname)s %(message)s')
logger.info("Start setting DEP configuration")

APP_CONFIG_FOLDER = "/mnt/data/app_config"

# check IPAC application mode
mode = "standard" #default app mode

if not os.path.exists(APP_CONFIG_FOLDER):
    logger.info(f"{APP_CONFIG_FOLDER} does not exist... creating")
    os.makedirs(APP_CONFIG_FOLDER)

if not os.path.isfile(f"{APP_CONFIG_FOLDER}/application.json"):
    logger.info(f"{APP_CONFIG_FOLDER}/application.json does not exist ... use default")
    shutil.copyfile("/barix/config/application.json", "/mnt/data/app_config/application.json")

with open(f"{APP_CONFIG_FOLDER}/application.json") as f:
    appCfg = json.load(f)
    if "application" in appCfg and "mode" in appCfg["application"] :
        mode = appCfg["application"]["mode"]

logger.info(f"IPAC application mode: {mode}")

if mode == "aes67":
    uci = Uci()
    if uci.get("dante","dep","enabled") != "false":
        logger.info("Disabling DEP")
        uci.set("dante","dep","enabled","false")
        uci.commit("dante")

    sys.exit(0)

elif mode == "dante" or mode == "standard": # Using DEP
    
    # check if "Dante" feature is available
    with open('/tmp/factory_info_partition/factory_info_json') as f:
        res = json.load(f)
        for elem in res['SW_LICENSES']:
            if 'Image_Features' in elem:
                if "Dante" in elem['Image_Features']:
                    logger.info("Dante feature is available")
                    break
                else: 
                    logger.info("Dante feature is not present")
                    sys.exit(0)

    if mode == "standard":
        uci = Uci()
        uci.set("dante","dep","enabled","true")
        uci.set("dante","dep","tx_channels","2")
        uci.set("dante","dep","rx_channels","2")
        logger.info("Enabling DEP - transmit/receiver")
        uci.set("dante","dante_app","dep_audio_interface","DANTE_ALSA_PLUGIN")
        uci.set("dante","dante_app","alsa_out","hw:0,0,4")
        logger.info("Dante App settings - DEP interface: DANTE_ALSA_PLUGIN, alsa out: hw:0,0,4")
        uci.commit("dante")
    else: # dante
        hwType = int(get_hw_type())

        uci = Uci()
        uci.set("dante","dep","enabled","true")
        uci.set("dante","dep","tx_channels","0")
        uci.set("dante","dep","rx_channels","2")
        logger.info("Enabling DEP - receiver only")
        uci.set("dante","dante_app","dep_audio_interface","DANTE_BUFFER_API")
        if hwType in [BarixHWType.Barix_MPA400.value, BarixHWType.Hyperspike_MPA400.value, BarixHWType.Barix_TPA400.value]: 
            uci.set("dante","dante_app","alsa_out","downmix_digital")
            logger.info("Dante App settings - DEP interface: DANTE_BUFFER_API, alsa out: downmix_digital")
        elif hwType in [BarixHWType.Barix_MPI400.value, BarixHWType.Barix_MR400.value]:
            uci.set("dante","dante_app","alsa_out","downmix_analog")
            logger.info("Dante App settings - DEP interface: DANTE_BUFFER_API, alsa out: downmix_analog")
        else:
            uci.set("dante","dante_app","alsa_out","hw:2,0")
            logger.info("Dante App settings - DEP interface: DANTE_BUFFER_API, alsa out: hw:2,0")
        uci.commit("dante")
        
else:
    logger.info("Invalid application mode!")
